import { createRouteRef, useApi, configApiRef, errorApiRef, createPlugin, createComponentExtension } from '@backstage/core-plugin-api';
import React, { createContext, useContext, useState, useEffect, Component, useMemo } from 'react';
import { union, literal, object, string, optional, array, number, parse, ValiError, undefined_, record } from 'valibot';
import { scmIntegrationsApiRef } from '@backstage/integration-react';
import { useEntity, getEntitySourceLocation } from '@backstage/plugin-catalog-react';
import { useQuery, useQueryClient, QueryClient, QueryClientProvider } from '@tanstack/react-query';

const rootRouteRef = createRouteRef({
  id: "coder"
});

const workspaceAgentStatusSchema = union([
  literal("connected"),
  literal("connecting"),
  literal("disconnected"),
  literal("timeout")
]);
const workspaceAgentSchema = object({
  id: string(),
  status: workspaceAgentStatusSchema
});
const workspaceResourceSchema = object({
  id: string(),
  agents: optional(array(workspaceAgentSchema))
});
const workspaceStatusSchema = union([
  literal("canceled"),
  literal("canceling"),
  literal("deleted"),
  literal("deleting"),
  literal("failed"),
  literal("pending"),
  literal("running"),
  literal("starting"),
  literal("stopped"),
  literal("stopping")
]);
const workspaceBuildSchema = object({
  id: string(),
  resources: array(workspaceResourceSchema),
  status: workspaceStatusSchema
});
const workspaceSchema = object({
  id: string(),
  name: string(),
  template_icon: string(),
  owner_name: string(),
  latest_build: workspaceBuildSchema
});
const workspaceBuildParameterSchema = object({
  name: string(),
  value: string()
});
const workspaceBuildParametersSchema = array(
  workspaceBuildParameterSchema
);
const workspacesResponseSchema = object({
  count: number(),
  workspaces: array(workspaceSchema)
});

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _response;
const CODER_QUERY_KEY_PREFIX = "coder-backstage-plugin";
const PROXY_ROUTE_PREFIX = "/api/proxy/coder";
const API_ROUTE_PREFIX = `${PROXY_ROUTE_PREFIX}/api/v2`;
const ASSETS_ROUTE_PREFIX = PROXY_ROUTE_PREFIX;
const CODER_AUTH_HEADER_KEY = "Coder-Session-Token";
const REQUEST_TIMEOUT_MS = 2e4;
function getCoderApiRequestInit(authToken) {
  return {
    headers: { [CODER_AUTH_HEADER_KEY]: authToken },
    signal: AbortSignal.timeout(REQUEST_TIMEOUT_MS)
  };
}
class BackstageHttpError extends Error {
  constructor(errorMessage, response) {
    super(errorMessage);
    __privateAdd(this, _response, void 0);
    this.name = "HttpError";
    __privateSet(this, _response, response);
  }
  get status() {
    return __privateGet(this, _response).status;
  }
  get ok() {
    return __privateGet(this, _response).ok;
  }
  get contentType() {
    return __privateGet(this, _response).headers.get("content_type");
  }
}
_response = new WeakMap();
async function getWorkspaces(fetchInputs) {
  var _a;
  const { baseUrl, coderQuery, auth } = fetchInputs;
  assertValidCoderAuth(auth);
  const urlParams = new URLSearchParams({
    q: coderQuery,
    limit: "0"
  });
  const response = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/workspaces?${urlParams.toString()}`,
    getCoderApiRequestInit(auth.token)
  );
  if (!response.ok) {
    throw new BackstageHttpError(
      `Unable to retrieve workspaces for query (${coderQuery})`,
      response
    );
  }
  if (!((_a = response.headers.get("content-type")) == null ? void 0 : _a.includes("application/json"))) {
    throw new BackstageHttpError(
      "200 request has no data - potential proxy issue",
      response
    );
  }
  const json = await response.json();
  const { workspaces: workspaces2 } = parse(workspacesResponseSchema, json);
  const withRemappedImgUrls = workspaces2.map((ws) => {
    const templateIcon = ws.template_icon;
    if (!templateIcon.startsWith("/")) {
      return ws;
    }
    return {
      ...ws,
      template_icon: `${baseUrl}${ASSETS_ROUTE_PREFIX}${templateIcon}`
    };
  });
  return withRemappedImgUrls;
}
async function getWorkspaceBuildParameters(inputs) {
  var _a;
  const { baseUrl, auth, workspaceBuildId } = inputs;
  assertValidCoderAuth(auth);
  const res = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/workspacebuilds/${workspaceBuildId}/parameters`,
    getCoderApiRequestInit(auth.token)
  );
  if (!res.ok) {
    throw new BackstageHttpError(
      `Failed to retreive build params for workspace ID ${workspaceBuildId}`,
      res
    );
  }
  if (!((_a = res.headers.get("content-type")) == null ? void 0 : _a.includes("application/json"))) {
    throw new BackstageHttpError(
      "200 request has no data - potential proxy issue",
      res
    );
  }
  const json = await res.json();
  return parse(workspaceBuildParametersSchema, json);
}
async function getWorkspacesByRepo(inputs) {
  const workspaces2 = await getWorkspaces(inputs);
  const paramResults = await Promise.allSettled(
    workspaces2.map(
      (ws) => getWorkspaceBuildParameters({
        ...inputs,
        workspaceBuildId: ws.latest_build.id
      })
    )
  );
  const { repoConfig } = inputs;
  const matchedWorkspaces = [];
  for (const [index, res] of paramResults.entries()) {
    if (res.status === "rejected") {
      continue;
    }
    for (const param of res.value) {
      const include = repoConfig.repoUrlParamKeys.includes(param.name) && param.value === repoConfig.repoUrl;
      if (include) {
        matchedWorkspaces.push(workspaces2[index]);
        break;
      }
    }
  }
  return matchedWorkspaces;
}
function serializeWorkspaceUrl(config, coderAccessUrl) {
  var _a;
  const formattedParams = new URLSearchParams({
    mode: (_a = config.mode) != null ? _a : "manual"
  });
  const unformatted = config.params;
  if (unformatted !== void 0 && unformatted.hasOwnProperty) {
    for (const key in unformatted) {
      if (!unformatted.hasOwnProperty(key)) {
        continue;
      }
      const value = unformatted[key];
      if (value !== void 0) {
        formattedParams.append(`param.${key}`, value);
      }
    }
  }
  const safeTemplate = encodeURIComponent(config.templateName);
  return `${coderAccessUrl}/templates/${safeTemplate}/workspace?${formattedParams.toString()}`;
}
function getWorkspaceAgentStatuses(workspace) {
  const uniqueStatuses = [];
  for (const resource of workspace.latest_build.resources) {
    if (resource.agents === void 0) {
      continue;
    }
    for (const agent of resource.agents) {
      const status = agent.status;
      if (!uniqueStatuses.includes(status)) {
        uniqueStatuses.push(status);
      }
    }
  }
  return uniqueStatuses;
}
function isWorkspaceOnline(workspace) {
  const latestBuildStatus = workspace.latest_build.status;
  const isAvailable = latestBuildStatus !== "stopped" && latestBuildStatus !== "stopping" && latestBuildStatus !== "pending";
  if (!isAvailable) {
    return false;
  }
  const statuses = getWorkspaceAgentStatuses(workspace);
  return statuses.every(
    (status) => status === "connected" || status === "connecting"
  );
}
function workspaces(inputs) {
  const enabled = inputs.auth.status === "authenticated";
  return {
    queryKey: [CODER_QUERY_KEY_PREFIX, "workspaces", inputs.coderQuery],
    queryFn: () => getWorkspaces(inputs),
    enabled,
    keepPreviousData: enabled && inputs.coderQuery !== ""
  };
}
function workspacesByRepo(inputs) {
  const enabled = inputs.auth.status === "authenticated" && inputs.coderQuery !== "";
  return {
    queryKey: [CODER_QUERY_KEY_PREFIX, "workspaces", inputs.coderQuery, "repo"],
    queryFn: () => getWorkspacesByRepo(inputs),
    enabled,
    keepPreviousData: enabled
  };
}
async function isAuthValid$1(inputs) {
  const { baseUrl, authToken } = inputs;
  const response = await fetch(
    `${baseUrl}${API_ROUTE_PREFIX}/users/me`,
    getCoderApiRequestInit(authToken)
  );
  if (response.status >= 400 && response.status !== 401) {
    throw new BackstageHttpError("Failed to complete request", response);
  }
  return response.status !== 401;
}
const authQueryKey = [CODER_QUERY_KEY_PREFIX, "auth"];
function authValidation(inputs) {
  const enabled = inputs.authToken !== "";
  return {
    queryKey: [...authQueryKey, inputs.authToken],
    queryFn: () => isAuthValid$1(inputs),
    enabled,
    keepPreviousData: enabled
  };
}

function useBackstageEndpoints() {
  const backstageConfig = useApi(configApiRef);
  const baseUrl = backstageConfig.getString("backend.baseUrl");
  return {
    baseUrl,
    assetsProxyUrl: `${baseUrl}${ASSETS_ROUTE_PREFIX}`,
    apiProxyUrl: `${baseUrl}${API_ROUTE_PREFIX}`
  };
}

const TOKEN_STORAGE_KEY = "coder-backstage-plugin/token";
const AUTH_GRACE_PERIOD_TIMEOUT_MS = 6e3;
function isAuthValid(state) {
  return state.status === "authenticated" || state.status === "distrustedWithGracePeriod";
}
function assertValidCoderAuth(auth) {
  if (!isAuthValid(auth)) {
    throw new Error("Coder auth is not valid");
  }
}
const AuthContext = createContext(null);
function useCoderAuth() {
  const contextValue = useContext(AuthContext);
  if (contextValue === null) {
    throw new Error(
      `Hook ${useCoderAuth.name} is being called outside of CoderProvider`
    );
  }
  return contextValue;
}
const CoderAuthProvider = ({ children }) => {
  const { baseUrl } = useBackstageEndpoints();
  const [isInsideGracePeriod, setIsInsideGracePeriod] = useState(true);
  const [authToken, setAuthToken] = useState(readAuthToken);
  const [readonlyInitialAuthToken] = useState(authToken);
  const authValidityQuery = useQuery({
    ...authValidation({ baseUrl, authToken }),
    refetchOnWindowFocus: (query) => query.state.data !== false
  });
  const authState = generateAuthState({
    authToken,
    authValidityQuery,
    isInsideGracePeriod,
    initialAuthToken: readonlyInitialAuthToken
  });
  if (!isInsideGracePeriod && authState.status === "authenticated") {
    setIsInsideGracePeriod(true);
  }
  useEffect(() => {
    if (authState.status === "authenticated") {
      window.localStorage.setItem(TOKEN_STORAGE_KEY, authState.token);
    }
  }, [authState.status, authState.token]);
  useEffect(() => {
    if (authState.status !== "distrustedWithGracePeriod") {
      return void 0;
    }
    const distrustTimeoutId = window.setTimeout(() => {
      setIsInsideGracePeriod(false);
    }, AUTH_GRACE_PERIOD_TIMEOUT_MS);
    return () => window.clearTimeout(distrustTimeoutId);
  }, [authState.status]);
  const queryClient = useQueryClient();
  useEffect(() => {
    let isRefetchingTokenQuery = false;
    const queryCache = queryClient.getQueryCache();
    const unsubscribe = queryCache.subscribe(async (event) => {
      const queryError = event.query.state.error;
      const shouldRevalidate = !isRefetchingTokenQuery && queryError instanceof BackstageHttpError && queryError.status === 401;
      if (!shouldRevalidate) {
        return;
      }
      isRefetchingTokenQuery = true;
      await queryClient.refetchQueries({ queryKey: authQueryKey });
      isRefetchingTokenQuery = false;
    });
    return unsubscribe;
  }, [queryClient]);
  return /* @__PURE__ */ React.createElement(
    AuthContext.Provider,
    {
      value: {
        ...authState,
        isAuthenticated: isAuthValid(authState),
        tokenLoadedOnMount: readonlyInitialAuthToken !== "",
        registerNewToken: (newToken) => {
          if (newToken !== "") {
            setAuthToken(newToken);
          }
        },
        ejectToken: () => {
          window.localStorage.removeItem(TOKEN_STORAGE_KEY);
          queryClient.removeQueries({ queryKey: [CODER_QUERY_KEY_PREFIX] });
          setAuthToken("");
        }
      }
    },
    children
  );
};
function generateAuthState({
  authToken,
  initialAuthToken,
  authValidityQuery,
  isInsideGracePeriod
}) {
  const isInitializing = initialAuthToken !== "" && authValidityQuery.isLoading && authValidityQuery.isFetching && !authValidityQuery.isFetchedAfterMount;
  if (isInitializing) {
    return {
      status: "initializing",
      token: void 0,
      error: void 0
    };
  }
  if (!authToken) {
    return {
      status: "tokenMissing",
      token: void 0,
      error: void 0
    };
  }
  if (authValidityQuery.error instanceof BackstageHttpError) {
    const deploymentLikelyUnavailable = authValidityQuery.error.status === 504 || authValidityQuery.error.status === 200 && authValidityQuery.error.contentType !== "application/json";
    if (deploymentLikelyUnavailable) {
      return {
        status: "deploymentUnavailable",
        token: void 0,
        error: authValidityQuery.error
      };
    }
  }
  const isTokenValidFromPrevFetch = authValidityQuery.data === true;
  if (isTokenValidFromPrevFetch) {
    const canTrustAuthThisRender = authValidityQuery.isSuccess && !authValidityQuery.isPaused;
    if (canTrustAuthThisRender) {
      return {
        status: "authenticated",
        token: authToken,
        error: void 0
      };
    }
    if (isInsideGracePeriod) {
      return {
        status: "distrustedWithGracePeriod",
        token: authToken,
        error: void 0
      };
    }
    return {
      status: "distrusted",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  const isAuthenticating = authValidityQuery.isLoading || authValidityQuery.isRefetching && (authValidityQuery.isError && authValidityQuery.data !== true || authValidityQuery.isSuccess && authValidityQuery.data === false);
  if (isAuthenticating) {
    return {
      status: "authenticating",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  const isCoderDeploymentDown = authValidityQuery.error instanceof Error && authValidityQuery.error.name === "TimeoutError";
  if (isCoderDeploymentDown) {
    return {
      status: "distrusted",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  if (authValidityQuery.isPaused) {
    return {
      status: "noInternetConnection",
      token: void 0,
      error: authValidityQuery.error
    };
  }
  return {
    status: "invalid",
    token: void 0,
    error: authValidityQuery.error
  };
}
function readAuthToken() {
  var _a;
  return (_a = window.localStorage.getItem(TOKEN_STORAGE_KEY)) != null ? _a : "";
}

const AppConfigContext = createContext(null);
function useCoderAppConfig() {
  const value = useContext(AppConfigContext);
  if (value === null) {
    throw new Error(
      `Hook ${useCoderAppConfig.name} must be called from a CoderProvider component`
    );
  }
  return value;
}
const CoderAppConfigProvider = ({
  children,
  appConfig
}) => {
  return /* @__PURE__ */ React.createElement(AppConfigContext.Provider, { value: appConfig }, children);
};

const FallbackUi = () => {
  return /* @__PURE__ */ React.createElement("p", null, "Error encountered. Please check browser console.");
};
class ErrorBoundaryCore extends Component {
  static getDerivedStateFromError() {
    return { hasError: true };
  }
  constructor(props) {
    super(props);
    this.state = { hasError: false };
  }
  componentDidCatch(error, errorInfo) {
    this.props.onError(error, errorInfo.componentStack || "no details");
  }
  render() {
    const { children, fallbackUi } = this.props;
    return this.state.hasError ? fallbackUi : children;
  }
}
const CoderErrorBoundary$1 = ({
  children,
  fallbackUi
}) => {
  const errorApi = useApi(errorApiRef);
  const fallbackContent = fallbackUi != null ? fallbackUi : /* @__PURE__ */ React.createElement(FallbackUi, null);
  const onError = (error, componentStack) => {
    var _a;
    let errorToLog;
    if (error instanceof ValiError) {
      const logs = error.issues.map((issue) => issue.message).join(", ");
      errorToLog = new Error(logs);
    } else {
      errorToLog = new Error();
      errorToLog.message = error.message;
      errorToLog.name = error.name;
    }
    errorToLog.stack = `${(_a = error.stack) != null ? _a : ""}

React Component stack:
${componentStack}`;
    errorApi.post(errorToLog, {
      hidden: false
    });
  };
  return /* @__PURE__ */ React.createElement(ErrorBoundaryCore, { fallbackUi: fallbackContent, onError }, children);
};

const MAX_FETCH_FAILURES = 3;
const shouldRetryRequest = (failureCount, error) => {
  const isBelowThreshold = failureCount < MAX_FETCH_FAILURES;
  if (!(error instanceof BackstageHttpError)) {
    return isBelowThreshold;
  }
  const isAuthenticationError = error.status === 401;
  const isLikelyProxyConfigurationError = error.status === 504 || error.status === 200 && error.contentType !== "application/json";
  return !isAuthenticationError && !isLikelyProxyConfigurationError && isBelowThreshold;
};
const defaultClient = new QueryClient({
  defaultOptions: {
    queries: {
      retry: shouldRetryRequest
    },
    mutations: {
      retry: shouldRetryRequest
    }
  }
});
const CoderProvider$1 = ({
  children,
  appConfig,
  queryClient = defaultClient
}) => {
  return /* @__PURE__ */ React.createElement(CoderErrorBoundary$1, null, /* @__PURE__ */ React.createElement(QueryClientProvider, { client: queryClient }, /* @__PURE__ */ React.createElement(CoderAppConfigProvider, { appConfig }, /* @__PURE__ */ React.createElement(CoderAuthProvider, null, children))));
};

const yamlConfigSchema = union([
  undefined_(),
  object({
    templateName: optional(string()),
    mode: optional(
      union(
        [literal("manual"), literal("auto")],
        "If defined, createMode must be 'manual' or 'auto'"
      )
    ),
    params: optional(
      record(
        string(),
        // Defining record value with undefined case as a safety net if user
        // hasn't or can't turn on the noUncheckedIndexedAccess compiler option
        union([string(), undefined_()]),
        "If defined, params must be JSON-serializable as Record<string, string>"
      )
    )
  })
]);
function compileCoderConfig(workspaceSettings, rawYamlConfig, repoUrl) {
  var _a, _b, _c, _d;
  const compiledParams = {};
  const yamlConfig = parse(yamlConfigSchema, rawYamlConfig);
  const paramsPrecedence = [workspaceSettings.params, (_a = yamlConfig == null ? void 0 : yamlConfig.params) != null ? _a : {}];
  for (const params of paramsPrecedence) {
    for (const key in params) {
      if (params.hasOwnProperty(key) && typeof params[key] === "string") {
        compiledParams[key] = params[key];
      }
    }
  }
  let cleanedUrl = repoUrl;
  if (repoUrl !== void 0) {
    cleanedUrl = repoUrl.replace(/\/tree\/main\/?$/, "");
    for (const key of workspaceSettings.repoUrlParamKeys) {
      compiledParams[key] = cleanedUrl;
    }
  }
  return {
    repoUrl: cleanedUrl,
    repoUrlParamKeys: workspaceSettings.repoUrlParamKeys,
    params: compiledParams,
    templateName: (_b = yamlConfig == null ? void 0 : yamlConfig.templateName) != null ? _b : workspaceSettings.templateName,
    mode: (_d = (_c = yamlConfig == null ? void 0 : yamlConfig.mode) != null ? _c : workspaceSettings.mode) != null ? _d : "manual"
  };
}
function useCoderEntityConfig() {
  var _a;
  const { entity } = useEntity();
  const appConfig = useCoderAppConfig();
  const sourceControlApi = useApi(scmIntegrationsApiRef);
  const rawYamlConfig = (_a = entity.spec) == null ? void 0 : _a.coder;
  const repoData = getEntitySourceLocation(entity, sourceControlApi);
  return useMemo(() => {
    return compileCoderConfig(
      appConfig.workspaces,
      rawYamlConfig,
      repoData == null ? void 0 : repoData.locationTargetUrl
    );
  }, [appConfig.workspaces, rawYamlConfig, repoData == null ? void 0 : repoData.locationTargetUrl]);
}

function useCoderWorkspaces(coderQuery, options) {
  const auth = useCoderAuth();
  const { baseUrl } = useBackstageEndpoints();
  const { repoConfig } = options != null ? options : {};
  const queryOptions = repoConfig ? workspacesByRepo({ coderQuery, auth, baseUrl, repoConfig }) : workspaces({ coderQuery, auth, baseUrl });
  return useQuery(queryOptions);
}

const coderPlugin = createPlugin({
  id: "coder",
  routes: {
    root: rootRouteRef
  }
});
const CoderProvider = coderPlugin.provide(
  createComponentExtension({
    name: "CoderAuthWrapper",
    component: {
      lazy: () => import('./index-c25694c1.esm.js').then((m) => m.CoderProvider)
    }
  })
);
const CoderAuthWrapper = coderPlugin.provide(
  createComponentExtension({
    name: "CoderAuthWrapper",
    component: {
      lazy: () => import('./index-f489806a.esm.js').then((m) => m.CoderAuthWrapper)
    }
  })
);
const CoderErrorBoundary = coderPlugin.provide(
  createComponentExtension({
    name: "CoderAuthWrapper",
    component: {
      lazy: () => import('./index-bac31e4b.esm.js').then(
        (m) => m.CoderErrorBoundary
      )
    }
  })
);
const CoderWorkspacesCard = coderPlugin.provide(
  createComponentExtension({
    name: "CoderAuthWrapper",
    component: {
      lazy: () => import('./index-68123abb.esm.js').then(
        (m) => m.CoderWorkspacesCard
      )
    }
  })
);

export { AuthContext as A, CoderAuthProvider as C, assertValidCoderAuth as a, useCoderAppConfig as b, CoderAppConfigProvider as c, CoderProvider$1 as d, CoderErrorBoundary$1 as e, useCoderWorkspaces as f, useCoderEntityConfig as g, useBackstageEndpoints as h, isWorkspaceOnline as i, coderPlugin as j, CoderProvider as k, CoderAuthWrapper as l, CoderErrorBoundary as m, CoderWorkspacesCard as n, serializeWorkspaceUrl as s, useCoderAuth as u };
//# sourceMappingURL=index-7cafdfe6.esm.js.map
